
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#if !defined (__SUGAR_CONTROLLERS_H_INSIDE__) && !defined (SUGAR_TOOLKIT_COMPILATION)
#error "Only <sugar/event-controller/sugar-event-controllers.h> can be included directly."
#endif

#ifndef __SUGAR_TYPE_BUILTINS_H__
#define __SUGAR_TYPE_BUILTINS_H__

#include <glib-object.h>

G_BEGIN_DECLS

/* enumerations from "sugar-event-controller.h" */
GType sugar_event_controller_state_get_type (void) G_GNUC_CONST;
#define SUGAR_TYPE_EVENT_CONTROLLER_STATE (sugar_event_controller_state_get_type ())
GType sugar_event_controller_flags_get_type (void) G_GNUC_CONST;
#define SUGAR_TYPE_EVENT_CONTROLLER_FLAGS (sugar_event_controller_flags_get_type ())

/* enumerations from "sugar-swipe-controller.h" */
GType sugar_swipe_direction_get_type (void) G_GNUC_CONST;
#define SUGAR_TYPE_SWIPE_DIRECTION (sugar_swipe_direction_get_type ())
GType sugar_swipe_direction_flags_get_type (void) G_GNUC_CONST;
#define SUGAR_TYPE_SWIPE_DIRECTION_FLAGS (sugar_swipe_direction_flags_get_type ())
G_END_DECLS

#endif /* __SUGAR_TYPE_BUILTINS_H__ */

/* Generated data ends here */

